"ForkBatch" <- function(run=Rolexa.env, path, outpath="./", prefix="rs_",
                        nthreads=3, nfiles=2, lane=1, tiles=1:100,...) {
    if (!is.loaded("fork")) stop("Need fork package")
    if (missing(run)) {
        run = SetModel(...)
    } else {
        run = SetModel(run,...)
    }
    pid = rep(0,nthreads)
    nbatch = nfiles*nthreads
    len = length(tiles)
    len2 = (len %/% nbatch)*nbatch
    len3 = (len %% nbatch) %/% nfiles
    len4 = (len %% nbatch) %% nfiles
    
    if (len2 >= nbatch) {
        for (fn in seq(1,len2,by=nbatch)) {
            for (th in seq(along=pid)) {
                the_fct = function() OneBatch(run,path,lane,tiles[fn+nfiles*th-nfiles:1],outpath,prefix)
                pid[th] = fork(the_fct)
            }
            for (p in pid) wait(p)
        }
    }
    if (len3) {
        for (fn in seq(len2,len2+len3*nfiles-1,by=nfiles)) {
            OneBatch(run,path,lane,tiles[fn+1:nfiles],outpath,prefix)
        }
    }
    if (len4) OneBatch(run,path,lane,tiles[1+len-len4:1],outpath,prefix)
}

setGeneric("OneBatch",
           function(run,...)
           standardGeneric("OneBatch"),
           signature = "run")
setMethod("OneBatch",
          signature = signature(run = "RolexaRun"),
          function(run=Rolexa.env, path, lane, tiles, outpath, prefix)
          {
              patt0 = paste("(",paste(sprintf("%04i",tiles),collapse="|"),")",sep='')
              pattern = paste("s_",lane,"_",patt0,"*",sep='')
              int = readIntensities(dirPath=path,pattern=pattern,withVariability=FALSE)
              pattern = paste("s_",lane,"_",patt0,"_seq*",sep='')
              seq = CombineReads(run=run,path=path,pattern=pattern)
              if (length(seq) == 0) {
                  seq = readFastq(path)
                  I = which(.IdToCoordinates(run@idsep,seq)$coord[,3] %in% tiles)
                  seq = seq[I]
              }
              ncycles = run@SequencingLength
              if (any(run@decorrelate == .decor.opts[c(1,3)])) {
                  theta = OptimizeAngle(int=int,cycles=1:ncycles)
                  int = DeCorrelateChannels(int=int,cycles=1:ncycles,theta=theta)
                  if (run@verbose>0) print(paste("theta=",theta))
              }
              if (any(run@decorrelate == .decor.opts[c(1,2)])) {
                  rate = OptimizeRate(int=int,ncycles=ncycles)
                  int = DeCorrelateCycles(int=int,ncycles=ncycles,rate=rate)
                  if (run@verbose>0) print(paste("rate=",rate))
              }
              if (run@normal) int = TileNormalize(run=run,int=int,cycles=1:ncycles)
              if (length(seq)) {
                  results = SeqScore(run=run,int=int,seqInit=seq,colonies=1:dim(int)[1],cycles=1:ncycles)
              } else {
                  results = SeqScore(run=run,int=int,colonies=1:dim(int)[1],cycles=1:ncycles)
              }
              results = FilterResults(run,results)
              if (length(results$sread)) SaveResults(run,results,outpath,prefix)
          })

