setGeneric("CombineFastQ",
           function(run,path,...)
           standardGeneric("CombineFastQ"),
           signature = c("run","path"))
setMethod("CombineFastQ",
          signature = signature(run="RolexaRun",path = "SolexaPath"),
          function(run=Rolexa.env,path,pattern="s_[1-8]_0[01][0-9]*",sext="_seq*",pext="_prb*")
          {
              dir = baseCallPath(path)
              id = readXStringColumns(dir, pattern=paste(pattern,sext,sep=''),
                colClasses=c(rep(list("BString"), 4), list(NULL)))
              name = strsplit(experimentPath(path),split='/')[[1]]
              seq = new("ShortReadQ",
                id = BStringSet(apply(cbind(name[length(name)],
                  as.character(id[[1]]),
                  as.character(id[[2]]),
                  as.character(id[[3]]),
                  as.character(id[[4]])),1,paste,collapse=run@idsep)),
                sread = readXStringColumns(dir, pattern=paste(pattern,sext,sep=''),
                  colClasses=c(rep(list(NULL), 4), list("DNAString")))[[1]],
                quality = readPrb(dir, pattern=paste(pattern,pext,sep='')))
              seq
          })


setGeneric("CombineReads",
           function(run,path,...)
           standardGeneric("CombineReads"),
           signature = c("run","path"))
setMethod("CombineReads",
          signature = signature(run="RolexaRun",path = "SolexaPath"),
          function(run=Rolexa.env,path,pattern="s_[1-8]_0[01][0-9]*_seq*")
          {
              dir = baseCallPath(path)
              id = readXStringColumns(dir, pattern=pattern,
                colClasses=c(rep(list("BString"), 4), list(NULL)))
              name = strsplit(experimentPath(path),split='/')[[1]]
              seq = new("ShortRead",
                id = BStringSet(apply(cbind(name[length(name)],
                  as.character(id[[1]]),
                  as.character(id[[2]]),
                  as.character(id[[3]]),
                  as.character(id[[4]])),1,paste,collapse=run@idsep)),
                sread = readXStringColumns(dir, pattern=pattern,
                  colClasses=c(rep(list(NULL), 4), list("DNAString")))[[1]])
              seq  
          })

".IdToCoordinates" <- function(split,seq,PET=FALSE) {
    c = matrix(unlist(strsplit(as.character(id(seq)),split=split)),
      nrow=length(seq),byrow=TRUE)
    n = ncol(c)
    if (n<4) stop("bad ID vector")
    cn = matrix(as.numeric(c[,(n-3):n]),ncol=4)
    name = c[1,1]
    if (n>4) name = paste(c[1,1:(n-4)],1,collapse=split)
    list(name=name,coord=cn)
}

".MatchCoordinates" <- function(split,seq,int) {
    scoord = .IdToCoordinates(split,seq)$coord
    icoord = cbind(readInfo(int)$lane,readInfo(int)$tile,readInfo(int)$x,readInfo(int)$y)
    Ii = which(apply(icoord,1,paste,collapse=":") %in% apply(scoord,1,paste,collapse=":"))
    Is = which(apply(scoord,1,paste,collapse=":") %in% apply(icoord,1,paste,collapse=":"))
    OIi = Ii[order(icoord[Ii,1],icoord[Ii,2],icoord[Ii,3],icoord[Ii,4])]
    OIs = Is[order(scoord[Is,1],scoord[Is,2],scoord[Is,3],scoord[Is,4])]
    cbind(OIi,OIs)
}

setGeneric("SaveResults",
           function(run,...)
           standardGeneric("SaveResults"),
           signature = "run")
setMethod("SaveResults",
          signature = signature(run="RolexaRun"),
          function(run=Rolexa.env,results,outpath,prefix="rs_")
          {
              lane = results$id[1,2]
              tiles = unique(results$id[1,3])
              if (length(tiles)>1) {
                  tlst =  paste(range(tiles),collapse="-")
              } else {
                  tlst = tiles[1]
              }
              file = paste(outpath,"/",prefix,lane,"_",tlst,".txt",sep='')
              I = which(results$entropy < 0)
              ## heuristic: puts entropy on a similar scale to fastq
              q1 = round(10*2^(2-results$entropy)+64)
              q1[I] = 0
              quality = apply(q1,1,intToUtf8)
              out = new("ShortReadQ",
                id=BStringSet(apply(results$id,1,paste,collapse=run@idsep)),
                sread=results$sread,
                quality=SFastqQuality(quality))
              writeFastq(out,file)
          })
