\name{TileImage} 

\alias{TileImage} 
\alias{TileImage,SolexaIntensity-method}
\title{Reconstruct tile image}
\description{Generate an image of the local intensity average}
\usage{
\S4method{TileImage}{SolexaIntensity}(int,cycle,tile,channel=c('A','C','G','T'),ncell=30)
TileImage(int,...)
}
\arguments{
  \item{int}{a \code{\link[ShortRead:SolexaIntensity-class]{SolexaIntensity}}
    object}
  \item{cycle}{the cycle to make an image of}
  \item{tile}{the tile to make an image of}
  \item{channel}{the channel ('A', 'C', 'G' or 'T') to make an image of}
  \item{ncell}{the number of divisions in each dimension for the image}
  \item{\dots}{additional arguments, ignored}
}
\details{\code{TileImage} creates an image of the intensity on a tile,
  in a given channel and at a given cycle. The tile is divided into
  \code{ncell*ncell} cells and the average intensity in each cell is
  represented on a color scale.
}
\references{Probabilistic base calling of Solexa sequencing data,
  BMC Bioinformatics 2008, 9:431}
\author{Jacques Rougemont, Arnaud Amzallag, Christian Iseli, Laurent Farinelli, Ioannis Xenarios, Felix Naef}
\examples{
path = SolexaPath(system.file("extdata", package="ShortRead"))
rolenv = SetModel(idsep="_")
int = readIntensities(path,pattern="s_1_0001",withVariability=FALSE)
par(mfrow=c(2,2))
for (c in c('A','C','G','T'))
    TileImage(int=int,cycle=1,tile=readInfo(int)$tile[1],channel=c,ncell=5)
int2 = TileNormalize(rolenv,int=int,cycles=1)
x11()
par(mfrow=c(2,2))
for (c in c('A','C','G','T'))
    TileImage(int=int2,cycle=1,tile=readInfo(int)$tile[1],channel=c,ncell=5)
}
\keyword{loess}
\keyword{hplot}
%

